package cz.fi.muni.pa165;


import cz.fi.muni.pa165.entity.*;
import cz.fi.muni.pa165.enums.BandType;
import cz.fi.muni.pa165.enums.MusicianType;
import cz.fi.muni.pa165.enums.UserRole;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

@Component
public class DataLoader implements ApplicationRunner {

    private final MusicianService musicianService;
    private final OfferService offerService;
    private final BandService bandService;
    private final UserService userService;
    private final ActivityService activityService;

    @Autowired
    public DataLoader(MusicianService musicianService, OfferService offerService, BandService bandService, UserService userService, ActivityService activityService) {
        this.musicianService = musicianService;
        this.offerService = offerService;
        this.bandService = bandService;
        this.userService = userService;
        this.activityService = activityService;
    }

    @Override
    @Transactional
    public void run(ApplicationArguments args) {
        createMusicians();
        createBand();
        createUser();
        createActivity();
    }

    public void createMusicians(){
        Musician musicianEric = new Musician();
        musicianEric.setId("0");
        musicianEric.setType(MusicianType.GUITARIST);
        musicianEric.setName("Eric Stone");
        musicianService.create(musicianEric);

        Musician musicianLea = new Musician();
        musicianLea.setName("Lea Lee");
        musicianLea.setType(MusicianType.SINGER);
        musicianLea.setId("1");
        musicianService.create(musicianLea);

        Musician musicianTadeas = new Musician();
        musicianTadeas.setName("Tadeas Smith");
        musicianTadeas.setType(MusicianType.DRUMMER);
        musicianTadeas.setId("2");
        musicianService.create(musicianTadeas);

        Musician musicianRebecca = new Musician();
        musicianRebecca.setName("Rebecca Fischer");
        musicianRebecca.setType(MusicianType.PIANIST);
        musicianRebecca.setId("3");
        musicianService.create(musicianRebecca);

        Musician musicianMatt = new Musician();
        musicianMatt.setName("Matthew Tuck");
        musicianMatt.setType(MusicianType.SINGER);
        musicianMatt.setId("4");
        musicianService.create(musicianMatt);

        Musician musicianMichael = new Musician();
        musicianMichael.setName("Michael Paget");
        musicianMichael.setType(MusicianType.GUITARIST);
        musicianMichael.setId("5");
        musicianService.create(musicianMichael);

        Musician musicianJamie = new Musician();
        musicianJamie.setName("Jamie Mathias");
        musicianJamie.setType(MusicianType.BASS_GUITARIST);
        musicianJamie.setId("6");
        musicianService.create(musicianJamie);

        Musician musicianJason = new Musician();
        musicianJason.setName("Jason Bowld");
        musicianJason.setType(MusicianType.DRUMMER);
        musicianJason.setId("7");
        musicianService.create(musicianJason);
    }

    public void createBand(){
        Band bandBFMV  = new Band();
        bandBFMV.setId("0");
        bandBFMV.setName("Bullet For My Valentine");
        bandBFMV.setType(BandType.METAL);
        bandBFMV.setLogo("BFMV");

        Musician matt = musicianService.findById("4");
        if (matt == null){
            //   log.warn("Musician Matthew Tuck does not exists!");
            throw new AssertionError();
        }
        bandBFMV.addMusician(matt);

        Musician michael = musicianService.findById("5");
        if (michael == null){
            //    log.warn("Musician Michael Paget does not exists!");
            throw new AssertionError();
        }
        bandBFMV.addMusician(michael);

        Musician jamie = musicianService.findById("6");
        if (jamie == null){
            //    log.warn("Musician Jamie Mathias does not exists!");
            throw new AssertionError();
        }
        bandBFMV.addMusician(jamie);

        Musician jason = musicianService.findById("7");
        if (jason == null){
            //  log.warn("Musician Jason Bowld does not exists!");
            throw new AssertionError();
        }
        bandBFMV.addMusician(jason);

        bandService.create(bandBFMV);

        Band bottleOfWatter = new Band();
        bottleOfWatter.setName("Bottle of Watter");
        bottleOfWatter.setType(BandType.PUNK);
        bottleOfWatter.setId("1");
        bandService.create(bottleOfWatter);

        Offer offer = new Offer();
        offer.setId("1");
        offer.setBand(bottleOfWatter);
        offer.setName("offer");
        offerService.create(offer);
    }


    public void createUser(){
        User user = new User();
        user.setName("Peter");
        user.setPassword("blabla");
        user.setUserRole(UserRole.MUSICIAN);
        user.setMusician(this.musicianService.findById("0"));
        userService.create(user);

        User userManager = new User();
        userManager.setName("Igor");
        userManager.setPassword("tralala");
        userManager.setUserRole(UserRole.MANAGER);
        userService.create(userManager);
    }

    public void createActivity(){
        Activity activity = new Activity();
        activity.setName("Rock for People");
        Band bfmv  = bandService.findById("0");
        if (bfmv == null){
            throw new AssertionError();
        }
        activity.setBand(bfmv);
        activity.setId("0");
        activity.setTime(LocalDateTime.of(2022, 6, 15, 0, 0));
        activityService.create(activity);

        Activity activity1 = new Activity();
        activity1.setTime(LocalDateTime.of(2022, 7,5, 0,0));
        activity1.setName("Concert");
        activity1.setId("1");
        activityService.create(activity1);
    }


}
